#include <sstream>
#include "AircraftCarrier.h"

/*
Función: Constructor de la clase AircraftCarrier
name: Nombre del portaviones
*/
AircraftCarrier::AircraftCarrier(string name){
    if(name==""){
        throw invalid_argument("Wrong name");
    }
    this->name=name;
    wins=0;
    losses=0;
}

/*
Función que añade cazas al portaviones
fd: Cadena con los cazas a añadir
return: Nada
*/
void AircraftCarrier::addFighters(string fd){
    stringstream ss(fd);
    string token,type;
    int count;
    char slash;

    if(fd!=""){
        while(getline(ss,token,':')){
            stringstream tokenStream(token);

            tokenStream >> count >> slash >> type;

            for(int i=0;i<count;i++){
                Fighter *f=new Fighter(type,name);
                fleet.push_back(f);
            }
        }
    }
}

/*
Función que actualiza los resultados del portaviones
r: Resultado de la batalla
return: Nada
*/
void AircraftCarrier::updateResults(int r){
    if(r==1){
        wins++;
    }
    else if(r==-1){
        losses++;
    }
}

/*
Función que devuelve el primer caza disponible del portaviones
type: Tipo de caza
return: Puntero al primer caza disponible
*/
Fighter* AircraftCarrier::getFirstAvailableFighter(string type) const{
    Fighter* f=nullptr;
    bool found=false;

    for(unsigned int i=0;i<fleet.size() && !found;i++){
         if(!fleet[i]->isDestroyed() && !fleet[i]->getPosition().isValid() && (type=="" || fleet[i]->getType()==type)){
            f=fleet[i];
            found=true;
        }
    }

    return f;
}

/*
Función que elimina los cazas destruidos del portaviones
return: Número de cazas destruidos
*/
int AircraftCarrier::purgeFleet(){
    int destroyed=0;
    unsigned int i=0;

    while(i<fleet.size()){
        if(fleet[i]->isDestroyed()){
            delete fleet[i];
            fleet.erase(fleet.begin()+i);
            destroyed++;
        }
        else{
            i++;
        }
    }
    
    return destroyed; 
}

/*
Función que muestra los cazas del portaviones
return: Nada
*/
void AircraftCarrier::showFleet() const{
    for(unsigned int i=0;i<fleet.size();i++){
        cout << *fleet[i];
        if(fleet[i]->isDestroyed()){
            cout << " (X)";
        }
        cout << endl;
    }
}

/*
Función que devuelve una cadnea con los cazas del portaviones no destruidos en el formato "n/t:n/t:..."
return: Cadena con los cazas del portaviones
*/
string AircraftCarrier::myFleet() const{
    string result;
    vector<string> types;
    vector<int> counts;

    // Contar los cazas no destruidos
    for (unsigned int i=0;i<fleet.size();i++){
        if(!fleet[i]->isDestroyed()){
            bool found=false;
            for(unsigned int j=0;j<types.size() && !found;j++){
                if(types[j]==fleet[i]->getType()){
                    counts[j]++;
                    found=true;
                }
            }
            if(!found){
                types.push_back(fleet[i]->getType());
                counts.push_back(1);
            }
        }
    }

    // Construir la cadena resultante
    for (unsigned int i=0;i<types.size();i++){
        if(result!=""){
            result+=":";
        }
        result+=to_string(counts[i]) + "/" + types[i];
    }

    return result;
}

/*
Sobrecarga del operador de salida
os: Flujo de salida
a: Portaviones a mostrar
return: Flujo de salida
*/
ostream& operator<<(ostream &os, const AircraftCarrier &a){
    os << "Aircraft Carrier [" << a.name << " " << a.wins << "/" << a.losses << "] " << a.myFleet();

    return os;
}


