#include "Board.h"

/*
Constructor que inicializa el tablero
size: tamaño del tablero
*/
Board::Board(int size){
    if(size<=0){
        throw invalid_argument("Wrong size");
    }
    this->size=size;
    for(int i=0;i<size;i++){
        vector<Fighter*> v;
        for(int j=0;j<size;j++){
            Fighter* f=nullptr;
            v.push_back(f);
        }
        board.push_back(v);
    }
}

/*
Función que devuelve el caza en una posición
c: coordenada
return: Caza en la posición
*/
Fighter* Board::getFighter(Coordinate c) const{
    Fighter *fighter=nullptr;

    if(inside(c)){
        fighter=board[c.getRow()][c.getColumn()];
    }

    return fighter;
}

/*
Función que comprueba si una coordenada está dentro del tablero
c: coordenada
return: true si está dentro, false si no
*/
bool Board::inside(Coordinate c) const{
    return (c.getRow()>=0 && c.getRow()<size && c.getColumn()>=0 && c.getColumn()<size);
}

/*
Función que lanza un caza en una posición
c: coordenada
f: caza a lanzar
return: 0 si no se ha lanzado, 1 si se ha lanzado y ha ganado, -1 si se ha lanzado y ha perdido
*/
int Board::launch(Coordinate c, Fighter *f){
    int result=0;

    // Si la coordenada no está dentro del tablero, no hacemos nada.
    if(!inside(c)){
        result = 0;
    }
    else if(f==nullptr || f->getPosition().isValid()){
        // Si no hay caza para lanzar o ya está posicionado, se devuelve 0.
        result=0;
    }
    else {
        // Obtenemos el caza que actualmente ocupa la posición.
        Fighter *occupant=board[c.getRow()][c.getColumn()];

        if(occupant==nullptr){
            // Si la posición está vacía, se coloca el caza y se actualiza su posición.
            board[c.getRow()][c.getColumn()]=f;
            f->setPosition(c);
            result=0;
        }
        else if(occupant->getAircraftCarrier()==f->getAircraftCarrier()){
            // Si el caza en la posición pertenece al mismo portaaviones, no se hace nada.
            result=0;
        }
        else {
            // Hay un caza enemigo en la posición: se inicia el combate.
            int combateResultado=f->fight(occupant);

            if(combateResultado==1){
                // Si f gana (resultado positivo) se coloca en esa posición y se resetea la posición del enemigo.
                board[c.getRow()][c.getColumn()]=f;
                f->setPosition(c);
                occupant->resetPosition();
                result=1;
            }
            else {
                // Si gana el enemigo (resultado negativo), no se realizan cambios en las posiciones.
                result=-1;
            }
        }
    }

    return result;
}



