#include <cstdlib> // Para rand()
#include "Fighter.h"

int Fighter::nextId = 1;

/*
Función que devuelve un número aleatorio entre 0 y maxrnd-1
maxrnd: número máximo
return: número aleatorio
*/
int getRandomNumber(unsigned int maxrnd){
    return rand() % maxrnd;
}

/*
Constructor que inicializa un caza
type: tipo de caza
motherShip: nombre de la nave nodriza
*/
Fighter::Fighter(string type, string aircraftCarrier){
    if(type==""){
        throw invalid_argument("Wrong type");
    }
    this->type=type;
    this->aircraftCarrier=aircraftCarrier;
    speed=100;
    attack=80;
    shield=80;
    id=nextId++;
}

/*
Función que añade ataque al caza
at: ataque a añadir
return: void
*/
void Fighter::addAttack(int at){
    attack+=at;
    if(attack<0){
        attack=0;
    }
}

/*
Función que añade velocidad al caza
sp: velocidad a añadir
return: void
*/
void Fighter::addSpeed(int sp){
    speed+=sp;
    if(speed<0){
        speed=0;
    }
}

/*
Función que añade escudo al caza
sh: escudo a añadir
return: void
*/
void Fighter::addShield(int sh){
    shield+=sh;
}

/*
Función que devuelve el daño que hace el caza
n: número aleatorio
return: int
*/
int Fighter::getDamage(int n) const{
    return (n*attack/300);
}

/*
Función que simula una pelea entre dos cazas
enemy: caza enemigo
return: int
*/
int Fighter::fight(Fighter *enemy){
    int result=0;
    int n,v1,v2,u,damage;
    
    if(!isDestroyed() && !enemy->isDestroyed()){
        while(result==0){
            n=getRandomNumber(100);
            v1=speed;
            v2=enemy->getSpeed();
            u=(100*v1)/(v1+v2);

            if(u>=n){
                damage=getDamage(n);
                enemy->addShield(-damage);
                if(enemy->isDestroyed()){
                    result=1;
                }
            }
            else{
                damage=enemy->getDamage(100-n);
                addShield(-damage);
                if(isDestroyed()){
                    result=-1;
                }
            }
        }
    }

    return result;
}

/*
Función que devuelve si el caza está destruido
return: bool
*/
bool Fighter::isDestroyed() const{
    return shield <= 0;
}

/*
Función que imprime un caza con el formato "(tipo id posición {velocidad,ataque,escudo})"
os: flujo de salida
*/
ostream& operator<<(ostream &os, const Fighter &f){
    os << "(" << f.type << " " << f.id << " " << f.position << " {" << f.speed << "," << f.attack << "," << f.shield << "})";
    return os;
}